@@ -1,11 +1,12 @@
 /*
  * Copyright (C) 2009-2010, Pino Toscano <pino@kde.org>
- * Copyright (C) 2017-2019, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2017-2020, Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2017, Jason Alan Palmer <jalanpalmer@gmail.com>
- * Copyright (C) 2018, Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
- * Copyright (C) 2018, Adam Reichold <adam.reichold@t-online.de>
+ * Copyright (C) 2018, 2020, Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
+ * Copyright (C) 2018, 2020, Adam Reichold <adam.reichold@t-online.de>
  * Copyright (C) 2018, Zsombor Hollay-Horvath <hollay.horvath@gmail.com>
  * Copyright (C) 2018, Aleksey Nikolaev <nae202@gmail.com>
+ * Copyright (C) 2020, Jiri Jakes <freedesktop@jirijakes.eu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +32,8 @@
 #include "poppler-document-private.h"
 #include "poppler-page-private.h"
 #include "poppler-private.h"
+#include "poppler-font-private.h"
+#include "poppler-font.h"
 
 #include "TextOutputDev.h"
 
@@ -45,6 +48,7 @@ page_private::page_private(document_private *_doc, int _index)
     , page(doc->doc->getCatalog()->getPage(_index + 1))
     , index(_index)
     , transition(nullptr)
+    , font_info_cache_initialized(false)
 {
 }
 
@@ -53,6 +57,21 @@ page_private::~page_private()
     delete transition;
 }
 
+void page_private::init_font_info_cache()
+{
+    if (font_info_cache_initialized)
+	return;
+
+    poppler::font_iterator it(index, doc);
+
+    if (it.has_next()) {
+	font_info_cache = it.next();
+    }
+
+    font_info_cache_initialized = true;
+    return;
+}
+
 /**
  \class poppler::page poppler-page.h "poppler/cpp/poppler-page.h"
 
@@ -262,6 +281,10 @@ ustring page::text(const rectf &r) const
     return text(r, physical_layout);
 }
 
+static void appendToGooString(void *stream, const char *text, int len) {
+  ((GooString *) stream)->append(text, len);
+}
+
 /**
  Returns the text in the page.
 
@@ -275,24 +298,24 @@ ustring page::text(const rectf &r) const
  */
 ustring page::text(const rectf &r, text_layout_enum layout_mode) const
 {
-    std::unique_ptr<GooString> s;
+    std::unique_ptr<GooString> out(new GooString());
     const bool use_raw_order = (layout_mode == raw_order_layout);
-    TextOutputDev td(nullptr, false, 0, use_raw_order, false);
-    d->doc->doc->displayPage(&td, d->index + 1, 72, 72, 0, false, true, false);
+    const bool use_physical_layout = (layout_mode == physical_layout);
+    TextOutputDev td(&appendToGooString, out.get(), use_physical_layout, 0, use_raw_order, false);
     if (r.is_empty()) {
-        PDFRectangle rect = *d->page->getCropBox();
-        const int rotate = d->page->getRotate();
-        if (rotate == 90 || rotate == 270) {
-            std::swap(rect.x1, rect.y1);
-            std::swap(rect.x2, rect.y2);
-        }
-        s.reset(td.getText(rect.x1, rect.y1, rect.x2, rect.y2));
+        d->doc->doc->displayPage(&td, d->index + 1, 72, 72, 0, false, true, false);
     } else {
-        s.reset(td.getText(r.left(), r.top(), r.right(), r.bottom()));
+        d->doc->doc->displayPageSlice(&td, d->index + 1, 72, 72, 0, false, true, false,
+				      r.left(), r.top(), r.width(), r.height());
     }
-    return ustring::from_utf8(s->c_str());
+    return ustring::from_utf8(out->c_str());
 }
 
+/*
+ * text_box_font_info object for text_box
+ */
+text_box_font_info_data::~text_box_font_info_data() = default;
+
 /*
  * text_box object for page::text_list()
  */
@@ -334,17 +357,50 @@ bool text_box::has_space_after() const
     return m_data->has_space_after;
 }
 
-std::vector<text_box> page::text_list() const
+bool text_box::has_font_info() const
+{
+    return (m_data->text_box_font != nullptr);
+}
+
+text_box::writing_mode_enum text_box::get_wmode(int i) const
+{
+    if (this->has_font_info())
+        return m_data->text_box_font->wmodes[i];
+    else
+        return text_box::invalid_wmode;
+}
+
+double text_box::get_font_size() const
+{
+    if (this->has_font_info())
+        return m_data->text_box_font->font_size;
+    else
+        return -1;
+}
+
+std::string text_box::get_font_name(int i) const
+{
+    if (!this->has_font_info())
+        return std::string("*ignored*");
+
+    int j = m_data->text_box_font->glyph_to_cache_index[i];
+    if (j < 0) {
+        return std::string("");
+    }
+    return m_data->text_box_font->font_info_cache[j].name();
+}
+
+std::vector<text_box> page::text_list(int opt_flag) const
 {
     std::vector<text_box>  output_list;
 
     /* config values are same with Qt5 Page::TextList() */
     auto output_dev = std::make_unique<TextOutputDev>(
-	nullptr,    /* char* fileName */
-	false,  /* bool physLayoutA */
+	nullptr, /* char* fileName */
+	false,   /* bool physLayoutA */
 	0,       /* double fixedPitchA */
-	false,  /* bool rawOrderA */
-	false  /* bool append */
+	false,   /* bool rawOrderA */
+	false    /* bool append */
     );
 
     /*
@@ -378,18 +434,66 @@ std::vector<text_box> page::text_list() const
                 {xMin, yMin, xMax-xMin, yMax-yMin},
                 word->getRotation(),
                 {},
-                word->hasSpaceAfter() == true
+                word->hasSpaceAfter() == true,
+                nullptr
             }};
 
+            std::unique_ptr<text_box_font_info_data> tb_font_info = nullptr;
+            if (opt_flag & page::text_list_include_font) {
+                d->init_font_info_cache();
+
+                std::unique_ptr<text_box_font_info_data> tb_font{new text_box_font_info_data{
+                    word->getFontSize(), // double font_size
+                    {},                  // std::vector<text_box::writing_mode> wmodes;
+                    d->font_info_cache,  // std::vector<font_info> font_info_cache;
+                    {}                   // std::vector<int> glyph_to_cache_index;
+                }};
+
+                tb_font_info = std::move(tb_font);
+            };
+
             tb.m_data->char_bboxes.reserve(word->getLength());
             for (int j = 0; j < word->getLength(); j ++) {
                 word->getCharBBox(j, &xMin, &yMin, &xMax, &yMax);
                 tb.m_data->char_bboxes.emplace_back(xMin, yMin, xMax-xMin, yMax-yMin);
             }
 
+            if (tb_font_info && d->font_info_cache_initialized) {
+                tb_font_info->glyph_to_cache_index.reserve(word->getLength());
+                for (int j = 0; j < word->getLength(); j++) {
+                    const TextFontInfo* cur_text_font_info = word->getFontInfo(j);
+
+                    // filter-out the invalid WMode value here.
+                    switch (cur_text_font_info->getWMode()) {
+                    case 0:
+                        tb_font_info->wmodes.push_back(text_box::horizontal_wmode);
+                        break;
+                    case 1:
+                        tb_font_info->wmodes.push_back(text_box::vertical_wmode);
+                        break;
+                    default:
+                        tb_font_info->wmodes.push_back(text_box::invalid_wmode);
+                    };
+
+                    tb_font_info->glyph_to_cache_index.push_back(-1);
+                    for (size_t k = 0; k < tb_font_info->font_info_cache.size(); k++) {
+                        if (cur_text_font_info->matches(&(tb_font_info->font_info_cache[k].d->ref))) {
+                            tb_font_info->glyph_to_cache_index[j] = k;
+                            break;
+                        }
+                    }
+                }
+                tb.m_data->text_box_font = std::move(tb_font_info);
+            }
+
             output_list.push_back(std::move(tb));
         }
     }
 
     return output_list;
 }
+
+std::vector<text_box> page::text_list() const
+{
+    return text_list(0);
+}
