@@ -27,11 +27,14 @@
 #include <UnicodeMap.h>
 #include <GfxState.h>
 #include <PageTransition.h>
+#include <BBoxOutputDev.h>
 #endif
 
 #include "poppler.h"
 #include "poppler-private.h"
 
+#define SUPPORTED_ROTATION(r) (r == 90 || r == 180 || r == 270)
+
 /**
  * SECTION:poppler-page
  * @short_description: Information about a page in a document
@@ -1399,9 +1402,11 @@ poppler_page_get_annot_mapping (PopplerPage *page)
     PopplerRectangle rect;
     Annot *annot;
     PDFRectangle *annot_rect;
+    gboolean flag_no_rotate;
     gint rotation = 0;
 
     annot = annots->getAnnot (i);
+    flag_no_rotate = annot->getFlags () & Annot::flagNoRotate;
 
     /* Create the mapping */
     mapping = poppler_annot_mapping_new ();
@@ -1449,35 +1454,53 @@ poppler_page_get_annot_mapping (PopplerPage *page)
     rect.x2 = annot_rect->x2 - crop_box->x1;
     rect.y2 = annot_rect->y2 - crop_box->y1;
 
-    if (! (annot->getFlags () & Annot::flagNoRotate))
-      rotation = page->page->getRotate ();
+    rotation = page->page->getRotate ();
 
-    switch (rotation)
-      {
-      case 90:
-        mapping->area.x1 = rect.y1;
-        mapping->area.y1 = height - rect.x2;
-        mapping->area.x2 = mapping->area.x1 + (rect.y2 - rect.y1);
-        mapping->area.y2 = mapping->area.y1 + (rect.x2 - rect.x1);
-        break;
-      case 180:
-        mapping->area.x1 = width - rect.x2;
-        mapping->area.y1 = height - rect.y2;
-        mapping->area.x2 = mapping->area.x1 + (rect.x2 - rect.x1);
-        mapping->area.y2 = mapping->area.y1 + (rect.y2 - rect.y1);
-        break;
-      case 270:
-        mapping->area.x1 = width - rect.y2;
-        mapping->area.y1 = rect.x1;
-        mapping->area.x2 = mapping->area.x1 + (rect.y2 - rect.y1);
-        mapping->area.y2 = mapping->area.y1 + (rect.x2 - rect.x1);
-        break;
-      default:
-        mapping->area.x1 = rect.x1;
-        mapping->area.y1 = rect.y1;
-        mapping->area.x2 = rect.x2;
-        mapping->area.y2 = rect.y2;
+    if (rotation == 0 || !SUPPORTED_ROTATION (rotation)) { /* zero or unknown rotation */
+      mapping->area.x1 = rect.x1;
+      mapping->area.y1 = rect.y1;
+      mapping->area.x2 = rect.x2;
+      mapping->area.y2 = rect.y2;
+    } else {
+      gdouble annot_height = rect.y2 - rect.y1;
+      gdouble annot_width = rect.x2 - rect.x1;
+
+      if (flag_no_rotate) {
+        if (rotation == 90) {
+          mapping->area.x1 = rect.y2;
+          mapping->area.y1 = height - (rect.x1 + annot_height);
+          mapping->area.x2 = rect.y2 + annot_width;
+          mapping->area.y2 = height - rect.x1;
+        } else if (rotation == 180) {
+          mapping->area.x1 = width - rect.x1;
+          mapping->area.x2 = MIN (mapping->area.x1 + annot_width, width);
+          mapping->area.y2 = height - rect.y2;
+          mapping->area.y1 = MAX (0, mapping->area.y2 - annot_height);
+        } else if (rotation == 270) {
+          mapping->area.x1 = width - rect.y2;
+          mapping->area.x2 = MIN (mapping->area.x1 + annot_width, width);
+          mapping->area.y2 = rect.x1;
+          mapping->area.y1 = MAX (0, mapping->area.y2 - annot_height);
+        }
+      } else { /* !flag_no_rotate */
+        if (rotation == 90) {
+          mapping->area.x1 = rect.y1;
+          mapping->area.y1 = height - rect.x2;
+          mapping->area.x2 = mapping->area.x1 + annot_height;
+          mapping->area.y2 = mapping->area.y1 + annot_width;
+        } else if (rotation == 180) {
+          mapping->area.x1 = width - rect.x2;
+          mapping->area.y1 = height - rect.y2;
+          mapping->area.x2 = mapping->area.x1 + annot_width;
+          mapping->area.y2 = mapping->area.y1 + annot_height;
+        } else if (rotation == 270) {
+          mapping->area.x1 = width - rect.y2;
+          mapping->area.y1 = rect.x1;
+          mapping->area.x2 = mapping->area.x1 + annot_height;
+          mapping->area.y2 = mapping->area.y1 + annot_width;
+        }
       }
+    }
 
     map_list = g_list_prepend (map_list, mapping);
   }
@@ -2187,6 +2210,51 @@ poppler_page_get_crop_box (PopplerPage *page, PopplerRectangle *rect)
   rect->y2 = cropBox->y2;
 }
 
+/*
+ * poppler_page_get_bounding_box:
+ * @page: A #PopplerPage
+ * @rect: (out) return the bounding box of the page
+ *
+ * Returns the bounding box of the page, a rectangle enclosing all text, vector
+ * graphics (lines, rectangles and curves) and raster images in the page.
+ * Includes invisible text but not (yet) annotations like highlights and form
+ * elements.
+ *
+ * Return value: %TRUE if the page contains graphics, %FALSE otherwise
+ *
+ * Since: 0.88
+ */
+gboolean
+poppler_page_get_bounding_box (PopplerPage *page,
+                               PopplerRectangle *rect) {
+  BBoxOutputDev *bb_out;
+  bool hasGraphics;
+
+  g_return_val_if_fail(POPPLER_IS_PAGE (page), false);
+  g_return_val_if_fail(rect != nullptr, false);
+
+  bb_out = new BBoxOutputDev();
+
+  page->page->displaySlice(bb_out,
+                              72.0, 72.0, 0,
+                              false, /* useMediaBox */
+                              true, /* Crop */
+                              -1, -1, -1, -1,
+                              false, /* printing */
+                              nullptr, nullptr,
+			      nullptr, nullptr);
+  hasGraphics = bb_out->getHasGraphics();
+  if (hasGraphics) {
+    rect->x1 = bb_out->getX1();
+    rect->y1 = bb_out->getY1();
+    rect->x2 = bb_out->getX2();
+    rect->y2 = bb_out->getY2();
+  }
+
+  delete bb_out;
+  return hasGraphics;
+}
+
 /**
  * poppler_page_get_text_layout:
  * @page: A #PopplerPage
