@@ -23,10 +23,11 @@
 // Copyright 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
 // Copyright 2018 Chinmoy Ranjan Pradhan <chinmoyrp65@protonmail.com>
 // Copyright 2018 Adam Reichold <adam.reichold@t-online.de>
-// Copyright 2018, 2019 Nelson Benítez León <nbenitezl@gmail.com>
+// Copyright 2018-2020 Nelson Benítez León <nbenitezl@gmail.com>
 // Copyright 2019, 2020 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright 2019 Tomoyuki Kubota <himajin100000@gmail.com>
 // Copyright 2019 João Netto <joaonetto901@gmail.com>
+// Copyright 2020 Marek Kasik <mkasik@redhat.com>
 //
 //========================================================================
 
@@ -479,143 +480,14 @@ SignatureInfo *FormWidgetSignature::validateSignature(bool doVerifyCert, bool fo
   return static_cast<FormFieldSignature*>(field)->validateSignature(doVerifyCert, forceRevalidation, validationTime);
 }
 
-std::vector<Goffset> FormWidgetSignature::getSignedRangeBounds()
+std::vector<Goffset> FormWidgetSignature::getSignedRangeBounds() const
 {
-  Object* byteRangeObj = static_cast<FormFieldSignature*>(field)->getByteRange();
-  std::vector<Goffset> range_vec;
-  if (byteRangeObj->isArray())
-  {
-    if (byteRangeObj->arrayGetLength() == 4)
-    {
-      for (int i = 0; i < 2; ++i)
-      {
-        Object offsetObj(byteRangeObj->arrayGet(2*i));
-        Object lenObj(byteRangeObj->arrayGet(2*i+1));
-        if (offsetObj.isIntOrInt64() && lenObj.isIntOrInt64())
-        {
-          Goffset offset = offsetObj.getIntOrInt64();
-          Goffset len = lenObj.getIntOrInt64();
-          range_vec.push_back(offset);
-          range_vec.push_back(offset+len);
-        }
-      }
-    }
-  }
-  return range_vec;
+  return static_cast<FormFieldSignature*>(field)->getSignedRangeBounds();
 }
 
 GooString* FormWidgetSignature::getCheckedSignature(Goffset *checkedFileSize)
 {
-  Goffset start = 0;
-  Goffset end = 0;
-  const std::vector<Goffset> ranges = getSignedRangeBounds();
-  if (ranges.size() == 4)
-  {
-    start = ranges[1];
-    end = ranges[2];
-  }
-  if (end >= start+6)
-  {
-    BaseStream* stream = doc->getBaseStream();
-    *checkedFileSize = stream->getLength();
-    Goffset len = end-start;
-    stream->setPos(end-1);
-    int c2 = stream->lookChar();
-    stream->setPos(start);
-    int c1 = stream->getChar();
-    // PDF signatures are first ASN1 DER, then hex encoded PKCS#7 structures,
-    // possibly padded with 0 characters and enclosed in '<' and '>'.
-    // The ASN1 DER encoding of a PKCS#7 structure must start with the tag 0x30
-    // for SEQUENCE. The next byte must be 0x80 for ASN1 DER indefinite length
-    // encoding or (0x80 + n) for ASN1 DER definite length encoding
-    // where n is the number of subsequent "length bytes" which big-endian
-    // encode the length of the content of the SEQUENCE following them.
-    if (len <= std::numeric_limits<int>::max() && *checkedFileSize > end && c1 == '<' && c2 == '>')
-    {
-      GooString gstr;
-      ++start;
-      --end;
-      len = end-start;
-      Goffset pos = 0;
-      do
-      {
-        c1 = stream->getChar();
-        if (c1 == EOF)
-          return nullptr;
-        gstr.append(static_cast<char>(c1));
-      } while (++pos < len);
-      if (gstr.getChar(0) == '3' && gstr.getChar(1) == '0')
-      {
-        if (gstr.getChar(2) == '8' && gstr.getChar(3) == '0')
-        {
-          // ASN1 DER indefinite length encoding:
-          // We only check that all characters up to the enclosing '>'
-          // are hex characters and that there are two hex encoded 0 bytes
-          // just before the enclosing '>' marking the end of the indefinite
-          // length encoding.
-          int paddingCount = 0;
-          while (gstr.getChar(len-1) == '0' && gstr.getChar(len-2) == '0')
-          {
-            ++paddingCount;
-            len -= 2;
-          }
-          if (paddingCount < 2 || len%2 == 1)
-            len = 0;
-        }
-        else if (gstr.getChar(2) == '8')
-        {
-          // ASN1 DER definite length encoding:
-          // We calculate the length of the following bytes from the length bytes and
-          // check that after the length bytes and the following calculated number of
-          // bytes all bytes up to the enclosing '>' character are hex encoded 0 bytes.
-          int lenBytes = gstr.getChar(3) - '0';
-          if (lenBytes > 0 && lenBytes <= 4)
-          {
-            int sigLen = 0;
-            for (int i = 0; i < 2*lenBytes; ++i)
-            {
-              sigLen <<= 4;
-              char c = gstr.getChar(i+4);
-              if (isdigit(c))
-                sigLen += c - '0';
-              else if (isxdigit(c) && c >= 'a')
-                sigLen += c - 'a' + 10;
-              else if (isxdigit(c) && c >= 'A')
-                sigLen += c - 'A' + 10;
-              else
-              {
-                len = 0;
-                break;
-              }
-            }
-            if (sigLen > 0 && 2*(sigLen+lenBytes) <= len-4)
-            {
-              for (Goffset i = 2*(sigLen+lenBytes)+4; i < len; ++i)
-              {
-                if (gstr.getChar(i) != '0')
-                {
-                  len = 0;
-                  break;
-                }
-              }
-            }
-            else
-              len = 0;
-          }
-        }
-        for ( const char c : gstr.toStr() )
-        {
-          if (!isxdigit(c))
-            len = 0;
-        }
-        if (len > 0)
-        {
-          return new GooString(&gstr, 0, len);
-        }
-      }
-    }
-  }
-  return nullptr;
+  return static_cast<FormFieldSignature*>(field)->getCheckedSignature(checkedFileSize);
 }
 
 void FormWidgetSignature::updateWidgetAppearance()
@@ -646,6 +518,7 @@ FormField::FormField(PDFDoc *docA, Object &&aobj, const Ref aref, FormField *par
   fullyQualifiedName = nullptr;
   quadding = quaddingLeftJustified;
   hasQuadding = false;
+  standAlone = false;
 
   //childs
   Object obj1 = dict->lookup("Kids");
@@ -979,6 +852,67 @@ void FormField::setReadOnly (bool value)
   updateChildrenAppearance();
 }
 
+void FormField::reset(const std::vector<std::string>& excludedFields)
+{
+  resetChildren(excludedFields);
+}
+
+void FormField::resetChildren(const std::vector<std::string>& excludedFields)
+{
+  if (!terminal) {
+    for (int i = 0; i < numChildren; i++)
+      children[i]->reset(excludedFields);
+  }
+}
+
+bool FormField::isAmongExcludedFields(const std::vector<std::string>& excludedFields)
+{
+  Ref fieldRef;
+
+  for (const std::string &field : excludedFields) {
+    if (field.compare(field.size() - 2, 2, " R") == 0) {
+      if (sscanf(field.c_str(), "%d %d R", &fieldRef.num, &fieldRef.gen) == 2 &&
+          fieldRef == getRef())
+        return true;
+    } else {
+      if (field == getFullyQualifiedName()->toStr())
+        return true;
+    }
+  }
+
+  return false;
+}
+
+FormField* FormField::findFieldByRef (Ref aref)
+{
+  if (terminal) {
+    if (this->getRef() == aref)
+      return this;
+  } else {
+    for (int i = 0; i < numChildren; i++) {
+      FormField* result = children[i]->findFieldByRef(aref);
+      if (result)
+        return result;
+    }
+  }
+  return nullptr;
+}
+
+FormField* FormField::findFieldByFullyQualifiedName (const std::string &name)
+{
+  if (terminal) {
+    if (getFullyQualifiedName()->cmp(name.c_str()) == 0)
+      return this;
+  } else {
+    for (int i = 0; i < numChildren; i++) {
+      FormField* result = children[i]->findFieldByFullyQualifiedName(name);
+      if (result)
+        return result;
+    }
+  }
+  return nullptr;
+}
+
 //------------------------------------------------------------------------
 // FormFieldButton
 //------------------------------------------------------------------------
@@ -991,6 +925,7 @@ FormFieldButton::FormFieldButton(PDFDoc *docA, Object &&dictObj, const Ref refA,
   siblings = nullptr;
   numSiblings = 0;
   appearanceState.setToNull();
+  defaultAppearanceState.setToNull();
 
   btype = formButtonCheck;
   Object obj1 = Form::fieldLookup(dict, "Ff");
@@ -1017,6 +952,7 @@ FormFieldButton::FormFieldButton(PDFDoc *docA, Object &&dictObj, const Ref refA,
     // Even though V is inheritable we are interested in the value of this
     // field, if not present it's probably because it's a button in a set.
     appearanceState = dict->lookup("V");
+    defaultAppearanceState = Form::fieldLookup(dict, "DV");
   }
 }
 
@@ -1149,6 +1085,24 @@ FormFieldButton::~FormFieldButton()
     gfree(siblings);
 }
 
+void FormFieldButton::reset(const std::vector<std::string>& excludedFields)
+{
+  if (!isAmongExcludedFields(excludedFields)) {
+    if (getDefaultAppearanceState()) {
+      setState(getDefaultAppearanceState());
+    } else {
+      obj.getDict()->remove("V");
+
+      // Clear check button if it doesn't have default value.
+      // This behaviour is what Adobe Reader does, it is not written in specification.
+      if (btype == formButtonCheck)
+        setState("Off");
+    }
+  }
+
+  resetChildren(excludedFields);
+}
+
 //------------------------------------------------------------------------
 // FormFieldText
 //------------------------------------------------------------------------
@@ -1159,6 +1113,7 @@ FormFieldText::FormFieldText(PDFDoc *docA, Object &&dictObj, const Ref refA, For
   Object obj1;
   content = nullptr;
   internalContent = nullptr;
+  defaultContent = nullptr;
   multiline = password = fileSelect = doNotSpellCheck = doNotScroll = comb = richText = false;
   maxLen = 0;
 
@@ -1186,16 +1141,35 @@ FormFieldText::FormFieldText(PDFDoc *docA, Object &&dictObj, const Ref refA, For
     maxLen = obj1.getInt();
   }
 
-  obj1 = Form::fieldLookup(dict, "V");
+  fillContent(fillDefaultValue);
+  fillContent(fillValue);
+}
+
+void FormFieldText::fillContent(FillValueType fillType)
+{
+  Dict* dict = obj.getDict();
+  Object obj1;
+
+  obj1 = Form::fieldLookup(dict, fillType == fillDefaultValue ? "DV" : "V");
   if (obj1.isString()) {
     if (obj1.getString()->hasUnicodeMarker()) {
       if (obj1.getString()->getLength() > 2)
-        content = obj1.getString()->copy();
+        {
+          if (fillType == fillDefaultValue)
+            defaultContent = obj1.getString()->copy();
+          else
+            content = obj1.getString()->copy();
+        }
     } else if (obj1.getString()->getLength() > 0) {
       //non-unicode string -- assume pdfDocEncoding and try to convert to UTF16BE
       int tmp_length;
       char* tmp_str = pdfDocEncodingToUTF16(obj1.getString()->toStr(), &tmp_length);
-      content = new GooString(tmp_str, tmp_length);
+
+      if (fillType == fillDefaultValue)
+        defaultContent = new GooString(tmp_str, tmp_length);
+      else
+        content = new GooString(tmp_str, tmp_length);
+
       delete [] tmp_str;
     }
   }
@@ -1241,6 +1215,18 @@ FormFieldText::~FormFieldText()
 {
   delete content;
   delete internalContent;
+  delete defaultContent;
+}
+
+void FormFieldText::reset(const std::vector<std::string>& excludedFields)
+{
+  if (!isAmongExcludedFields(excludedFields)) {
+    setContentCopy(defaultContent);
+    if (defaultContent == nullptr)
+      obj.getDict()->remove("V");
+  }
+
+  resetChildren(excludedFields);
 }
 
 double FormFieldText::getTextFontSize()
@@ -1342,6 +1328,7 @@ FormFieldChoice::FormFieldChoice(PDFDoc *docA, Object &&aobj, const Ref refA, Fo
 {
   numChoices = 0;
   choices = nullptr;
+  defaultChoices = nullptr;
   editedChoice = nullptr;
   topIdx = 0;
 
@@ -1418,7 +1405,25 @@ FormFieldChoice::FormFieldChoice(PDFDoc *docA, Object &&aobj, const Ref refA, Fo
     // Note: According to PDF specs, /V should *never* contain the exportVal.
     // However, if /Opt is an array of (exportVal,optionName) pairs, acroread
     // seems to expect the exportVal instead of the optionName and so we do too.
-    obj1 = Form::fieldLookup(dict, "V");
+    fillChoices(fillValue);
+  }
+
+  fillChoices(fillDefaultValue);
+}
+
+void FormFieldChoice::fillChoices(FillValueType fillType)
+{
+  const char *key = fillType == fillDefaultValue ? "DV" : "V";
+  Dict* dict = obj.getDict();
+  Object obj1;
+
+  obj1 = Form::fieldLookup(dict, key);
+  if (obj1.isString() || obj1.isArray()) {
+    if (fillType == fillDefaultValue) {
+      defaultChoices = new bool[numChoices];
+      memset(defaultChoices, 0, sizeof(bool) * numChoices);
+    }
+
     if (obj1.isString()) {
       bool optionFound = false;
 
@@ -1434,13 +1439,16 @@ FormFieldChoice::FormFieldChoice(PDFDoc *docA, Object &&aobj, const Ref refA, Fo
         }
 
         if (optionFound) {
-          choices[i].selected = true;
+          if (fillType == fillDefaultValue)
+            defaultChoices[i] = true;
+          else
+            choices[i].selected = true;
           break; // We've determined that this option is selected. No need to keep on scanning
         }
       }
 
       // Set custom value if /V doesn't refer to any predefined option and the field is user-editable
-      if (!optionFound && edit) {
+      if (fillType == fillValue && !optionFound && edit) {
         editedChoice = obj1.getString()->copy();
       }
     } else if (obj1.isArray()) {
@@ -1448,7 +1456,7 @@ FormFieldChoice::FormFieldChoice(PDFDoc *docA, Object &&aobj, const Ref refA, Fo
         for (int j = 0; j < obj1.arrayGetLength(); j++) {
           const Object obj2 = obj1.arrayGet(j);
           if (!obj2.isString()) {
-            error(errSyntaxError, -1, "FormWidgetChoice:: V array contains a non string object");
+            error(errSyntaxError, -1, "FormWidgetChoice:: {0:s} array contains a non string object", key);
             continue;
           }
 
@@ -1465,7 +1473,10 @@ FormFieldChoice::FormFieldChoice(PDFDoc *docA, Object &&aobj, const Ref refA, Fo
           }
 
           if (matches) {
-            choices[i].selected = true;
+            if (fillType == fillDefaultValue)
+              defaultChoices[i] = true;
+            else
+              choices[i].selected = true;
             break; // We've determined that this option is selected. No need to keep on scanning
           }
         }
@@ -1481,6 +1492,7 @@ FormFieldChoice::~FormFieldChoice()
     delete choices[i].optionName;
   }
   delete [] choices;
+  delete [] defaultChoices;
   delete editedChoice;
 }
 
@@ -1631,6 +1643,25 @@ const GooString *FormFieldChoice::getSelectedChoice() const {
   return nullptr;
 }
 
+void FormFieldChoice::reset(const std::vector<std::string>& excludedFields)
+{
+  if (!isAmongExcludedFields(excludedFields)) {
+    delete editedChoice;
+    editedChoice = nullptr;
+
+    if (defaultChoices) {
+      for (int i = 0; i < numChoices; i++)
+        choices[i].selected = defaultChoices[i];
+    } else {
+      unselectAll();
+    }
+  }
+
+  resetChildren(excludedFields);
+
+  updateSelection();
+}
+
 //------------------------------------------------------------------------
 // FormFieldSignature
 //------------------------------------------------------------------------
@@ -1728,9 +1759,9 @@ void FormFieldSignature::hashSignedDataBlock(SignatureHandler *handler, Goffset
 #endif
 }
 
-FormSignatureType FormWidgetSignature::signatureType()
+FormSignatureType FormWidgetSignature::signatureType() const
 {
-  return static_cast<FormFieldSignature*>(field)->signature_type;
+  return static_cast<FormFieldSignature*>(field)->getSignatureType();
 }
 
 SignatureInfo *FormFieldSignature::validateSignature(bool doVerifyCert, bool forceRevalidation, time_t validationTime)
@@ -1811,6 +1842,144 @@ SignatureInfo *FormFieldSignature::validateSignature(bool doVerifyCert, bool for
   return signature_info;
 }
 
+std::vector<Goffset> FormFieldSignature::getSignedRangeBounds() const
+{
+  std::vector<Goffset> range_vec;
+  if (byte_range.isArray())
+  {
+    if (byte_range.arrayGetLength() == 4)
+    {
+      for (int i = 0; i < 2; ++i)
+      {
+        const Object offsetObj(byte_range.arrayGet(2*i));
+        const Object lenObj(byte_range.arrayGet(2*i+1));
+        if (offsetObj.isIntOrInt64() && lenObj.isIntOrInt64())
+        {
+          const Goffset offset = offsetObj.getIntOrInt64();
+          const Goffset len = lenObj.getIntOrInt64();
+          range_vec.push_back(offset);
+          range_vec.push_back(offset+len);
+        }
+      }
+    }
+  }
+  return range_vec;
+}
+
+GooString* FormFieldSignature::getCheckedSignature(Goffset *checkedFileSize)
+{
+  Goffset start = 0;
+  Goffset end = 0;
+  const std::vector<Goffset> ranges = getSignedRangeBounds();
+  if (ranges.size() == 4)
+  {
+    start = ranges[1];
+    end = ranges[2];
+  }
+  if (end >= start+6)
+  {
+    BaseStream* stream = doc->getBaseStream();
+    *checkedFileSize = stream->getLength();
+    Goffset len = end-start;
+    stream->setPos(end-1);
+    int c2 = stream->lookChar();
+    stream->setPos(start);
+    int c1 = stream->getChar();
+    // PDF signatures are first ASN1 DER, then hex encoded PKCS#7 structures,
+    // possibly padded with 0 characters and enclosed in '<' and '>'.
+    // The ASN1 DER encoding of a PKCS#7 structure must start with the tag 0x30
+    // for SEQUENCE. The next byte must be 0x80 for ASN1 DER indefinite length
+    // encoding or (0x80 + n) for ASN1 DER definite length encoding
+    // where n is the number of subsequent "length bytes" which big-endian
+    // encode the length of the content of the SEQUENCE following them.
+    if (len <= std::numeric_limits<int>::max() && *checkedFileSize > end && c1 == '<' && c2 == '>')
+    {
+      GooString gstr;
+      ++start;
+      --end;
+      len = end-start;
+      Goffset pos = 0;
+      do
+      {
+        c1 = stream->getChar();
+        if (c1 == EOF)
+          return nullptr;
+        gstr.append(static_cast<char>(c1));
+      } while (++pos < len);
+      if (gstr.getChar(0) == '3' && gstr.getChar(1) == '0')
+      {
+        if (gstr.getChar(2) == '8' && gstr.getChar(3) == '0')
+        {
+          // ASN1 DER indefinite length encoding:
+          // We only check that all characters up to the enclosing '>'
+          // are hex characters and that there are two hex encoded 0 bytes
+          // just before the enclosing '>' marking the end of the indefinite
+          // length encoding.
+          int paddingCount = 0;
+          while (gstr.getChar(len-1) == '0' && gstr.getChar(len-2) == '0')
+          {
+            ++paddingCount;
+            len -= 2;
+          }
+          if (paddingCount < 2 || len%2 == 1)
+            len = 0;
+        }
+        else if (gstr.getChar(2) == '8')
+        {
+          // ASN1 DER definite length encoding:
+          // We calculate the length of the following bytes from the length bytes and
+          // check that after the length bytes and the following calculated number of
+          // bytes all bytes up to the enclosing '>' character are hex encoded 0 bytes.
+          int lenBytes = gstr.getChar(3) - '0';
+          if (lenBytes > 0 && lenBytes <= 4)
+          {
+            int sigLen = 0;
+            for (int i = 0; i < 2*lenBytes; ++i)
+            {
+              sigLen <<= 4;
+              char c = gstr.getChar(i+4);
+              if (isdigit(c))
+                sigLen += c - '0';
+              else if (isxdigit(c) && c >= 'a')
+                sigLen += c - 'a' + 10;
+              else if (isxdigit(c) && c >= 'A')
+                sigLen += c - 'A' + 10;
+              else
+              {
+                len = 0;
+                break;
+              }
+            }
+            if (sigLen > 0 && 2*(sigLen+lenBytes) <= len-4)
+            {
+              for (Goffset i = 2*(sigLen+lenBytes)+4; i < len; ++i)
+              {
+                if (gstr.getChar(i) != '0')
+                {
+                  len = 0;
+                  break;
+                }
+              }
+            }
+            else
+              len = 0;
+          }
+        }
+        for ( const char c : gstr.toStr() )
+        {
+          if (!isxdigit(c))
+            len = 0;
+        }
+        if (len > 0)
+        {
+          return new GooString(&gstr, 0, len);
+        }
+      }
+    }
+  }
+  return nullptr;
+}
+
 void FormFieldSignature::print(int indent)
 {
   printf ("%*s- (%d %d): [signature] terminal: %s children: %d\n", indent, "", ref.num, ref.gen,
@@ -1983,6 +2152,56 @@ FormWidget* Form::findWidgetByRef (Ref aref)
   return nullptr;
 }
 
+FormField* Form::findFieldByRef(Ref aref) const
+{
+  for (int i = 0; i < numFields; i++) {
+    FormField *result = rootFields[i]->findFieldByRef(aref);
+    if (result) return result;
+  }
+  return nullptr;
+}
+
+FormField* Form::findFieldByFullyQualifiedName(const std::string &name) const
+{
+  for (int i = 0; i < numFields; i++) {
+    FormField *result = rootFields[i]->findFieldByFullyQualifiedName(name);
+    if (result) return result;
+  }
+  return nullptr;
+}
+
+void Form::reset (const std::vector<std::string>& fields, bool excludeFields)
+{
+  FormField  *foundField;
+  const bool  resetAllFields = fields.empty();
+
+  if (resetAllFields) {
+    for (int i = 0; i < numFields; i++) {
+      rootFields[i]->reset(std::vector<std::string>());
+    }
+  } else {
+    if (!excludeFields) {
+      for (const std::string &field : fields) {
+        Ref fieldRef;
+
+        if (field.compare(field.size() - 2, 2, " R") == 0 &&
+            sscanf(field.c_str(), "%d %d R", &fieldRef.num, &fieldRef.gen) == 2) {
+          foundField = findFieldByRef(fieldRef);
+        } else {
+          foundField = findFieldByFullyQualifiedName(field);
+        }
+
+        if (foundField)
+          foundField->reset(std::vector<std::string>());
+      }
+    } else {
+      for (int i = 0; i < numFields; i++) {
+        rootFields[i]->reset(fields);
+      }
+    }
+  }
+}
+
 //------------------------------------------------------------------------
 // FormPageWidgets
 //------------------------------------------------------------------------
@@ -2025,6 +2244,21 @@ FormPageWidgets::FormPageWidgets (Annots *annots, unsigned int page, Form *form)
   }
 }
 
+void FormPageWidgets::addWidgets(const std::vector<FormField*>& addedWidgets, unsigned int page)
+{
+  if (addedWidgets.empty())
+    return;
+
+  size += addedWidgets.size();
+  widgets = (FormWidget**)greallocn(widgets, size, sizeof(FormWidget*));
+
+  for (auto frmField : addedWidgets) {
+    FormWidget *frmWidget = frmField->getWidget(0);
+    frmWidget->setID(FormWidget::encodeID(page, numWidgets));
+    widgets[numWidgets++] = frmWidget;
+  }
+}
+
 FormPageWidgets::~FormPageWidgets()
 {
   gfree (widgets);
