@@ -6,7 +6,7 @@
 //
 // Copyright 2006 Julien Rebetez <julienr@svn.gnome.org>
 // Copyright 2007, 2008, 2011 Carlos Garcia Campos <carlosgc@gnome.org>
-// Copyright 2007-2010, 2012, 2015-2019 Albert Astals Cid <aacid@kde.org>
+// Copyright 2007-2010, 2012, 2015-2020 Albert Astals Cid <aacid@kde.org>
 // Copyright 2010 Mark Riedesel <mark@klowner.com>
 // Copyright 2011 Pino Toscano <pino@kde.org>
 // Copyright 2012 Fabio D'Urso <fabiodurso@hotmail.it>
@@ -20,6 +20,8 @@
 // Copyright 2018 Chinmoy Ranjan Pradhan <chinmoyrp65@protonmail.com>
 // Copyright 2019, 2020 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright 2019 João Netto <joaonetto901@gmail.com>
+// Copyright 2020 Nelson Benítez León <nbenitezl@gmail.com>
+// Copyright 2020 Marek Kasik <mkasik@redhat.com>
 //
 //========================================================================
 
@@ -72,6 +74,11 @@ enum FormSignatureType {
   ETSI_CAdES_detached
 };
 
+enum FillValueType {
+  fillValue,
+  fillDefaultValue
+};
+
 class Form;
 class FormField;
 class FormFieldButton;
@@ -228,6 +235,7 @@ public:
   int getNumChoices() const;
   //return the display name of the i-th choice (UTF16BE)
   const GooString* getChoice(int i) const;
+  const GooString* getExportVal(int i) const;
   //select the i-th choice
   void select (int i); 
 
@@ -266,13 +274,13 @@ public:
   FormWidgetSignature(PDFDoc *docA, Object *dictObj, unsigned num, Ref ref, FormField *p);
   void updateWidgetAppearance() override;
 
-  FormSignatureType signatureType();
+  FormSignatureType signatureType() const;
   // Use -1 for now as validationTime
   SignatureInfo *validateSignature(bool doVerifyCert, bool forceRevalidation, time_t validationTime);
 
   // returns a list with the boundaries of the signed ranges
   // the elements of the list are of type Goffset
-  std::vector<Goffset> getSignedRangeBounds();
+  std::vector<Goffset> getSignedRangeBounds() const;
 
   // checks the length encoding of the signature and returns the hex encoded signature
   // if the check passed (and the checked file size as output parameter in checkedFileSize)
@@ -302,6 +310,8 @@ public:
 
   void setReadOnly (bool value);
   bool isReadOnly () const { return readOnly; }
+  void setStandAlone (bool value) { standAlone = value; }
+  bool isStandAlone () const { return standAlone; }
 
   GooString* getDefaultAppearance() const { return defaultAppearance; }
   bool hasTextQuadding() const { return hasQuadding; }
@@ -314,8 +324,10 @@ public:
   GooString *getFullyQualifiedName();
 
   FormWidget* findWidgetByRef (Ref aref);
-  int getNumWidgets() { return terminal ? numChildren : 0; }
-  FormWidget *getWidget(int i) { return terminal ? widgets[i] : nullptr; }
+  int getNumWidgets() const { return terminal ? numChildren : 0; }
+  FormWidget *getWidget(int i) const { return terminal ? widgets[i] : nullptr; }
+  int getNumChildren() const { return !terminal ? numChildren : 0; }
+  FormField *getChildren(int i) const { return children[i]; }
 
   // only implemented in FormFieldButton
   virtual void fillChildrenSiblingsID ();
@@ -324,11 +336,16 @@ public:
 
   void printTree(int indent = 0);
   virtual void print(int indent = 0);
+  virtual void reset(const std::vector<std::string>& excludedFields);
+  void resetChildren(const std::vector<std::string>& excludedFields);
+  FormField* findFieldByRef(Ref aref);
+  FormField* findFieldByFullyQualifiedName(const std::string &name);
 
  protected:
   void _createWidget (Object *obj, Ref aref);
   void createChildren(std::set<int> *usedParents);
   void updateChildrenAppearance();
+  bool isAmongExcludedFields(const std::vector<std::string>& excludedFields);
 
   FormFieldType type;           // field type
   Ref ref;
@@ -352,6 +369,9 @@ public:
   bool hasQuadding;
   VariableTextQuadding quadding;
 
+  //True when FormField is not part of Catalog's Field array (or there isn't one).
+  bool standAlone;
+
 private:
   FormField() {}
 };
@@ -374,6 +394,7 @@ public:
   bool getState(const char *state) const;
 
   const char *getAppearanceState() const { return appearanceState.isName() ? appearanceState.getName() : nullptr; }
+  const char *getDefaultAppearanceState() const { return defaultAppearanceState.isName() ? defaultAppearanceState.getName() : nullptr; }
 
   void fillChildrenSiblingsID () override;
   
@@ -385,6 +406,7 @@ public:
   int getNumSiblings () const { return numSiblings; }
 
   void print(int indent) override;
+  void reset(const std::vector<std::string>& excludedFields) override;
 
   ~FormFieldButton() override;
 protected:
@@ -399,6 +421,7 @@ protected:
   int active_child; //only used for combo box
   bool noAllOff;
   Object appearanceState; // V
+  Object defaultAppearanceState; // DV
 };
 
 //------------------------------------------------------------------------
@@ -431,14 +454,17 @@ public:
   void setTextFontSize(int fontSize);
 
   void print(int indent) override;
+  void reset(const std::vector<std::string>& excludedFields) override;
 
   static int tokenizeDA(const GooString* daString, std::vector<GooString*>* daToks, const char* searchTok);
 
 protected:
   int parseDA(std::vector<GooString*>* daToks);
+  void fillContent(FillValueType fillType);
 
   GooString* content;
   GooString* internalContent;
+  GooString* defaultContent;
   bool multiline;
   bool password;
   bool fileSelect;
@@ -493,10 +519,12 @@ public:
   int getTopIndex() const { return topIdx; }
 
   void print(int indent) override;
+  void reset(const std::vector<std::string>& excludedFields) override;
 
 protected:
   void unselectAll();
   void updateSelection();
+  void fillChoices(FillValueType fillType);
 
   bool combo;
   bool edit;
@@ -512,6 +540,7 @@ protected:
 
   int numChoices;
   ChoiceOpt* choices;
+  bool* defaultChoices;
   GooString* editedChoice;
   int topIdx; // TI
 };
@@ -521,16 +550,25 @@ protected:
 //------------------------------------------------------------------------
 
 class FormFieldSignature: public FormField {
-  friend class FormWidgetSignature;
 public:
   FormFieldSignature(PDFDoc *docA, Object &&dict, const Ref ref, FormField *parent, std::set<int> *usedParents);
 
   // Use -1 for now as validationTime
   SignatureInfo *validateSignature(bool doVerifyCert, bool forceRevalidation, time_t validationTime);
 
+  // returns a list with the boundaries of the signed ranges
+  // the elements of the list are of type Goffset
+  std::vector<Goffset> getSignedRangeBounds() const;
+
+  // checks the length encoding of the signature and returns the hex encoded signature
+  // if the check passed (and the checked file size as output parameter in checkedFileSize)
+  // otherwise a nullptr is returned
+  GooString* getCheckedSignature(Goffset *checkedFileSize);
+
   ~FormFieldSignature() override;
   Object* getByteRange() { return &byte_range; }
   const GooString* getSignature() const { return signature; }
+  FormSignatureType getSignatureType() const { return signature_type; }
 
 private:
   void parseInfo();
@@ -563,7 +601,8 @@ public:
   static Object fieldLookup(Dict *field, const char *key);
   
   /* Creates a new Field of the type specified in obj's dict.
-     used in Form::Form and FormField::FormField */
+     used in Form::Form , FormField::FormField and
+     Page::loadStandaloneFields */
   static FormField *createFieldFromDict (Object &&obj, PDFDoc *docA, const Ref aref, FormField *parent, std::set<int> *usedParents);
 
   Object *getObj () const { return acroForm; }
@@ -576,11 +615,15 @@ public:
   Object* getDefaultResourcesObj() { return &resDict; }
 
   FormWidget* findWidgetByRef (Ref aref);
+  FormField* findFieldByRef(Ref aref) const;
+  FormField* findFieldByFullyQualifiedName(const std::string &name) const;
 
   void postWidgetsLoad();
 
   const std::vector<Ref> &getCalculateOrder() const { return calculateOrder; }
 
+  void reset(const std::vector<std::string>& fields, bool excludeFields);
+
 private:
   FormField** rootFields;
   int numFields;
@@ -612,6 +655,7 @@ public:
 
   int getNumWidgets() const { return numWidgets; }
   FormWidget* getWidget(int i) const { return widgets[i]; }
+  void addWidgets(const std::vector<FormField*>& addedWidgets, unsigned int page);
 
 private:
   FormWidget** widgets;
