@@ -22,9 +22,10 @@
 // Copyright (C) 2011 Andrea Canciani <ranma42@gmail.com>
 // Copyright (C) 2011-2014, 2016 Thomas Freitag <Thomas.Freitag@alfa.de>
 // Copyright (C) 2013 Lu Wang <coolwanglu@gmail.com>
-// Copyright (C) 2015, 2017 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2015, 2017, 2020 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2017, 2019 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
+// Copyright (C) 2020 Philipp Knechtges <philipp-dev@knechtges.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -39,8 +40,9 @@
 #include "Object.h"
 #include "Function.h"
 
-#include <assert.h>
+#include <cassert>
 #include <map>
+#include <memory>
 
 class Array;
 class Gfx;
@@ -142,6 +144,10 @@ struct GfxColor {
   GfxColorComp c[gfxColorMaxComps];
 };
 
+static inline void clearGfxColor(GfxColor *gfxColor) {
+  memset(gfxColor->c, 0, sizeof(GfxColorComp) * gfxColorMaxComps);
+}
+
 //------------------------------------------------------------------------
 // GfxGray
 //------------------------------------------------------------------------
@@ -184,27 +190,38 @@ enum GfxColorSpaceMode {
   csPattern
 };
 
+// This shall hold a cmsHPROFILE handle.
+// Only use the make_GfxLCMSProfilePtr function to construct this pointer,
+// to ensure that the resources are properly released after usage.
+typedef std::shared_ptr<void> GfxLCMSProfilePtr;
+
+#ifdef USE_CMS
+GfxLCMSProfilePtr make_GfxLCMSProfilePtr(void* profile);
+#endif
+
 // wrapper of cmsHTRANSFORM to copy
 class GfxColorTransform {
 public:
   void doTransform(void *in, void *out, unsigned int size);
   // transformA should be a cmsHTRANSFORM
-  GfxColorTransform(void *transformA, int cmsIntent, unsigned int inputPixelType, unsigned int transformPixelType);
+  GfxColorTransform(const GfxLCMSProfilePtr& sourceProfileA, void *transformA, int cmsIntent,
+                    unsigned int inputPixelType, unsigned int transformPixelType);
   ~GfxColorTransform();
   GfxColorTransform(const GfxColorTransform &) = delete;
   GfxColorTransform& operator=(const GfxColorTransform &) = delete;
   int getIntent() const { return cmsIntent; }
   int getInputPixelType() const { return inputPixelType; }
   int getTransformPixelType() const { return transformPixelType; }
-  void ref();
-  unsigned int unref();
+  GfxLCMSProfilePtr getSourceProfile() { return sourceProfile; }
+  char *getPostScriptCSA();
 private:
   GfxColorTransform() {}
+  GfxLCMSProfilePtr sourceProfile;
   void *transform;
-  unsigned int refCount;
   int cmsIntent;
   unsigned int inputPixelType;
   unsigned int transformPixelType;
+  char *psCSA;
 };
 
 class GfxColorSpace {
@@ -216,7 +233,7 @@ public:
   GfxColorSpace(const GfxColorSpace &) = delete;
   GfxColorSpace& operator=(const GfxColorSpace &other) = delete;
 
-  virtual GfxColorSpace *copy() = 0;
+  virtual GfxColorSpace *copy() const = 0;
   virtual GfxColorSpaceMode getMode() const = 0;
 
   // Construct a color space.  Returns nullptr if unsuccessful.
@@ -272,13 +289,10 @@ public:
 
 #ifdef USE_CMS
   static int setupColorProfiles();
-  // displayProfileA should be a cmsHPROFILE 
-  static void setDisplayProfile(void *displayProfileA);
+  static void setDisplayProfile(const GfxLCMSProfilePtr& displayProfileA);
   static void setDisplayProfileName(GooString *name);
-  // result will be a cmsHPROFILE 
-  static void *getRGBProfile();
-  // result will be a cmsHPROFILE 
-  static void *getDisplayProfile();
+  static GfxLCMSProfilePtr getRGBProfile();
+  static GfxLCMSProfilePtr getDisplayProfile();
 #endif
 protected:
 
@@ -294,8 +308,8 @@ class GfxDeviceGrayColorSpace: public GfxColorSpace {
 public:
 
   GfxDeviceGrayColorSpace();
-  ~GfxDeviceGrayColorSpace();
-  GfxColorSpace *copy() override;
+  ~GfxDeviceGrayColorSpace() override;
+  GfxColorSpace *copy() const override;
   GfxColorSpaceMode getMode() const override { return csDeviceGray; }
 
   void getGray(const GfxColor *color, GfxGray *gray) const override;
@@ -328,8 +342,8 @@ class GfxCalGrayColorSpace: public GfxColorSpace {
 public:
 
   GfxCalGrayColorSpace();
-  ~GfxCalGrayColorSpace();
-  GfxColorSpace *copy() override;
+  ~GfxCalGrayColorSpace() override;
+  GfxColorSpace *copy() const override;
   GfxColorSpaceMode getMode() const override { return csCalGray; }
 
   // Construct a CalGray color space.  Returns nullptr if unsuccessful.
@@ -360,7 +374,7 @@ private:
   double kr, kg, kb;		    // gamut mapping mulitpliers
   void getXYZ(const GfxColor *color, double *pX, double *pY, double *pZ) const;
 #ifdef USE_CMS
-  GfxColorTransform *transform;
+  std::shared_ptr<GfxColorTransform> transform;
 #endif
 };
 
@@ -372,8 +386,8 @@ class GfxDeviceRGBColorSpace: public GfxColorSpace {
 public:
 
   GfxDeviceRGBColorSpace();
-  ~GfxDeviceRGBColorSpace();
-  GfxColorSpace *copy() override;
+  ~GfxDeviceRGBColorSpace() override;
+  GfxColorSpace *copy() const override;
   GfxColorSpaceMode getMode() const override { return csDeviceRGB; }
 
   void getGray(const GfxColor *color, GfxGray *gray) const override;
@@ -406,8 +420,8 @@ class GfxCalRGBColorSpace: public GfxColorSpace {
 public:
 
   GfxCalRGBColorSpace();
-  ~GfxCalRGBColorSpace();
-  GfxColorSpace *copy() override;
+  ~GfxCalRGBColorSpace() override;
+  GfxColorSpace *copy() const override;
   GfxColorSpaceMode getMode() const override { return csCalRGB; }
 
   // Construct a CalRGB color space.  Returns nullptr if unsuccessful.
@@ -442,7 +456,7 @@ private:
   double kr, kg, kb;		    // gamut mapping mulitpliers
   void getXYZ(const GfxColor *color, double *pX, double *pY, double *pZ) const;
 #ifdef USE_CMS
-  GfxColorTransform *transform;
+  std::shared_ptr<GfxColorTransform> transform;
 #endif
 };
 
@@ -454,8 +468,8 @@ class GfxDeviceCMYKColorSpace: public GfxColorSpace {
 public:
 
   GfxDeviceCMYKColorSpace();
-  ~GfxDeviceCMYKColorSpace();
-  GfxColorSpace *copy() override;
+  ~GfxDeviceCMYKColorSpace() override;
+  GfxColorSpace *copy() const override;
   GfxColorSpaceMode getMode() const override { return csDeviceCMYK; }
 
   void getGray(const GfxColor *color, GfxGray *gray) const override;
@@ -485,8 +499,8 @@ class GfxLabColorSpace: public GfxColorSpace {
 public:
 
   GfxLabColorSpace();
-  ~GfxLabColorSpace();
-  GfxColorSpace *copy() override;
+  ~GfxLabColorSpace() override;
+  GfxColorSpace *copy() const override;
   GfxColorSpaceMode getMode() const override { return csLab; }
 
   // Construct a Lab color space.  Returns nullptr if unsuccessful.
@@ -523,7 +537,7 @@ private:
   double kr, kg, kb;		    // gamut mapping mulitpliers
   void getXYZ(const GfxColor *color, double *pX, double *pY, double *pZ) const;
 #ifdef USE_CMS
-  GfxColorTransform *transform;
+  std::shared_ptr<GfxColorTransform> transform;
 #endif
 };
 
@@ -534,9 +548,10 @@ private:
 class GfxICCBasedColorSpace: public GfxColorSpace {
 public:
 
-  GfxICCBasedColorSpace(int nCompsA, GfxColorSpace *altA);
-  ~GfxICCBasedColorSpace();
-  GfxColorSpace *copy() override;
+  GfxICCBasedColorSpace(int nCompsA, GfxColorSpace *altA,
+			const Ref *iccProfileStreamA);
+  ~GfxICCBasedColorSpace() override;
+  GfxColorSpace *copy() const override;
   GfxColorSpaceMode getMode() const override { return csICCBased; }
 
   // Construct an ICCBased color space.  Returns nullptr if unsuccessful.
@@ -564,6 +579,10 @@ public:
 
   // ICCBased-specific access.
   GfxColorSpace *getAlt() { return alt; }
+  Ref getRef() { return iccProfileStream; }
+#ifdef USE_CMS
+  char *getPostScriptCSA();
+#endif
 
 private:
 
@@ -571,10 +590,11 @@ private:
   GfxColorSpace *alt;		// alternate color space
   double rangeMin[4];		// min values for each component
   double rangeMax[4];		// max values for each component
+  Ref iccProfileStream;		// the ICC profile
 #ifdef USE_CMS
   int getIntent() { return (transform != nullptr) ? transform->getIntent() : 0; }
-  GfxColorTransform *transform;
-  GfxColorTransform *lineTransform; // color transform for line
+  std::shared_ptr<GfxColorTransform> transform;
+  std::shared_ptr<GfxColorTransform> lineTransform; // color transform for line
   mutable std::map<unsigned int, unsigned int> cmsCache;
 #endif
 };
@@ -586,8 +606,8 @@ class GfxIndexedColorSpace: public GfxColorSpace {
 public:
 
   GfxIndexedColorSpace(GfxColorSpace *baseA, int indexHighA);
-  ~GfxIndexedColorSpace();
-  GfxColorSpace *copy() override;
+  ~GfxIndexedColorSpace() override;
+  GfxColorSpace *copy() const override;
   GfxColorSpaceMode getMode() const override { return csIndexed; }
 
   // Construct an Indexed color space.  Returns nullptr if unsuccessful.
@@ -639,8 +659,8 @@ public:
 
   GfxSeparationColorSpace(GooString *nameA, GfxColorSpace *altA,
 			  Function *funcA);
-  ~GfxSeparationColorSpace();
-  GfxColorSpace *copy() override;
+  ~GfxSeparationColorSpace() override;
+  GfxColorSpace *copy() const override;
   GfxColorSpaceMode getMode() const override { return csSeparation; }
 
   // Construct a Separation color space.  Returns nullptr if unsuccessful.
@@ -684,8 +704,8 @@ public:
 
   GfxDeviceNColorSpace(int nCompsA, std::vector<std::string> &&namesA,
 		       GfxColorSpace *alt, Function *func, std::vector<GfxSeparationColorSpace*> *sepsCS);
-  ~GfxDeviceNColorSpace();
-  GfxColorSpace *copy() override;
+  ~GfxDeviceNColorSpace() override;
+  GfxColorSpace *copy() const override;
   GfxColorSpaceMode getMode() const override { return csDeviceN; }
 
   // Construct a DeviceN color space.  Returns nullptr if unsuccessful.
@@ -730,8 +750,8 @@ class GfxPatternColorSpace: public GfxColorSpace {
 public:
 
   GfxPatternColorSpace(GfxColorSpace *underA);
-  ~GfxPatternColorSpace();
-  GfxColorSpace *copy() override;
+  ~GfxPatternColorSpace() override;
+  GfxColorSpace *copy() const override;
   GfxColorSpaceMode getMode() const override { return csPattern; }
 
   // Construct a Pattern color space.  Returns nullptr if unsuccessful.
@@ -769,7 +789,7 @@ public:
 
   static GfxPattern *parse(GfxResources *res, Object *obj, OutputDev *out, GfxState *state, int patternRefNum);
 
-  virtual GfxPattern *copy() = 0;
+  virtual GfxPattern *copy() const = 0;
 
   int getType() const { return type; }
 
@@ -789,9 +809,9 @@ class GfxTilingPattern: public GfxPattern {
 public:
 
   static GfxTilingPattern *parse(Object *patObj, int patternRefNum);
-  ~GfxTilingPattern();
+  ~GfxTilingPattern() override;
 
-  GfxPattern *copy() override;
+  GfxPattern *copy() const override;
 
   int getPaintType() const { return paintType; }
   int getTilingType() const { return tilingType; }
@@ -807,8 +827,8 @@ private:
 
   GfxTilingPattern(int paintTypeA, int tilingTypeA,
 		   const double *bboxA, double xStepA, double yStepA,
-		   Object *resDictA, const double *matrixA,
-		   Object *contentStreamA, int patternRefNumA);
+		   const Object *resDictA, const double *matrixA,
+		   const Object *contentStreamA, int patternRefNumA);
 
   int paintType;
   int tilingType;
@@ -827,9 +847,9 @@ class GfxShadingPattern: public GfxPattern {
 public:
 
   static GfxShadingPattern *parse(GfxResources *res, Object *patObj, OutputDev *out, GfxState *state, int patternRefNum);
-  ~GfxShadingPattern();
+  ~GfxShadingPattern() override;
 
-  GfxPattern *copy() override;
+  GfxPattern *copy() const override;
 
   GfxShading *getShading() { return shading; }
   const double *getMatrix() const { return matrix; }
@@ -850,7 +870,7 @@ class GfxShading {
 public:
 
   GfxShading(int typeA);
-  GfxShading(GfxShading *shading);
+  GfxShading(const GfxShading *shading);
   virtual ~GfxShading();
 
   GfxShading(const GfxShading &) = delete;
@@ -858,7 +878,7 @@ public:
 
   static GfxShading *parse(GfxResources *res, Object *obj, OutputDev *out, GfxState *state);
 
-  virtual GfxShading *copy() = 0;
+  virtual GfxShading *copy() const = 0;
 
   int getType() const { return type; }
   GfxColorSpace *getColorSpace() { return colorSpace; }
@@ -898,8 +918,8 @@ public:
 		       double t0A, double t1A,
 		       Function **funcsA, int nFuncsA,
 		       bool extend0A, bool extend1A);
-  GfxUnivariateShading(GfxUnivariateShading *shading);
-  ~GfxUnivariateShading();
+  GfxUnivariateShading(const GfxUnivariateShading *shading);
+  ~GfxUnivariateShading() override;
 
   double getDomain0() const { return t0; }
   double getDomain1() const { return t1; }
@@ -919,7 +939,7 @@ public:
 				 double xMin, double yMin,
 				 double xMax, double yMax) = 0;
 
-  virtual double getDistance(double tMin, double tMax) const = 0;
+  virtual double getDistance(double sMin, double sMax) const = 0;
 
 private:
 
@@ -945,12 +965,12 @@ public:
 		     double x1A, double y1A,
 		     const double *matrixA,
 		     Function **funcsA, int nFuncsA);
-  GfxFunctionShading(GfxFunctionShading *shading);
-  ~GfxFunctionShading();
+  GfxFunctionShading(const GfxFunctionShading *shading);
+  ~GfxFunctionShading() override;
 
   static GfxFunctionShading *parse(GfxResources *res, Dict *dict, OutputDev *out, GfxState *state);
 
-  GfxShading *copy() override;
+  GfxShading *copy() const override;
 
   void getDomain(double *x0A, double *y0A, double *x1A, double *y1A) const
     { *x0A = x0; *y0A = y0; *x1A = x1; *y1A = y1; }
@@ -979,12 +999,12 @@ public:
 		  double t0A, double t1A,
 		  Function **funcsA, int nFuncsA,
 		  bool extend0A, bool extend1A);
-  GfxAxialShading(GfxAxialShading *shading);
-  ~GfxAxialShading();
+  GfxAxialShading(const GfxAxialShading *shading);
+  ~GfxAxialShading() override;
 
   static GfxAxialShading *parse(GfxResources *res, Dict *dict, OutputDev *out, GfxState *state);
 
-  GfxShading *copy() override;
+  GfxShading *copy() const override;
 
   void getCoords(double *x0A, double *y0A, double *x1A, double *y1A) const
     { *x0A = x0; *y0A = y0; *x1A = x1; *y1A = y1; }
@@ -993,7 +1013,7 @@ public:
 				 double xMin, double yMin,
 				 double xMax, double yMax) override;
 
-  double getDistance(double tMin, double tMax) const override;
+  double getDistance(double sMin, double sMax) const override;
 
 private:
 
@@ -1012,12 +1032,12 @@ public:
 		   double t0A, double t1A,
 		   Function **funcsA, int nFuncsA,
 		   bool extend0A, bool extend1A);
-  GfxRadialShading(GfxRadialShading *shading);
-  ~GfxRadialShading();
+  GfxRadialShading(const GfxRadialShading *shading);
+  ~GfxRadialShading() override;
 
   static GfxRadialShading *parse(GfxResources *res, Dict *dict, OutputDev *out, GfxState *state);
 
-  GfxShading *copy() override;
+  GfxShading *copy() const override;
 
   void getCoords(double *x0A, double *y0A, double *r0A,
 		 double *x1A, double *y1A, double *r1A) const
@@ -1027,7 +1047,7 @@ public:
 				 double xMin, double yMin,
 				 double xMax, double yMax) override;
 
-  double getDistance(double tMin, double tMax) const override;
+  double getDistance(double sMin, double sMax) const override;
 
 private:
 
@@ -1050,12 +1070,12 @@ public:
 			    GfxGouraudVertex *verticesA, int nVerticesA,
 			    int (*trianglesA)[3], int nTrianglesA,
 			    Function **funcsA, int nFuncsA);
-  GfxGouraudTriangleShading(GfxGouraudTriangleShading *shading);
-  ~GfxGouraudTriangleShading();
+  GfxGouraudTriangleShading(const GfxGouraudTriangleShading *shading);
+  ~GfxGouraudTriangleShading() override;
 
   static GfxGouraudTriangleShading *parse(GfxResources *res, int typeA, Dict *dict, Stream *str, OutputDev *out, GfxState *state);
 
-  GfxShading *copy() override;
+  GfxShading *copy() const override;
 
   int getNTriangles() const { return nTriangles; }
 
@@ -1136,12 +1156,12 @@ public:
 
   GfxPatchMeshShading(int typeA, GfxPatch *patchesA, int nPatchesA,
 		      Function **funcsA, int nFuncsA);
-  GfxPatchMeshShading(GfxPatchMeshShading *shading);
-  ~GfxPatchMeshShading();
+  GfxPatchMeshShading(const GfxPatchMeshShading *shading);
+  ~GfxPatchMeshShading() override;
 
   static GfxPatchMeshShading *parse(GfxResources *res, int typeA, Dict *dict, Stream *str, OutputDev *out, GfxState *state);
 
-  GfxShading *copy() override;
+  GfxShading *copy() const override;
 
   int getNPatches() const { return nPatches; }
   const GfxPatch *getPatch(int i) const { return &patches[i]; }
@@ -1185,7 +1205,7 @@ public:
   GfxImageColorMap& operator=(const GfxImageColorMap &) = delete;
 
   // Return a copy of this color map.
-  GfxImageColorMap *copy() { return new GfxImageColorMap(this); }
+  GfxImageColorMap *copy() const { return new GfxImageColorMap(this); }
 
   // Is color map valid?
   bool isOk() const { return ok; }
@@ -1223,7 +1243,7 @@ public:
   const GfxColor *getMatteColor() const { return (useMatte) ? &matteColor : nullptr; }
 private:
 
-  GfxImageColorMap(GfxImageColorMap *colorMap);
+  GfxImageColorMap(const GfxImageColorMap *colorMap);
 
   GfxColorSpace *colorSpace;	// the image color space
   int bits;			// bits per component
@@ -1585,9 +1605,10 @@ public:
     { strncpy(renderingIntent, intent, 31); }
 
 #ifdef USE_CMS
-  void setDisplayProfile(void *localDisplayProfileA);
-  void *getDisplayProfile() { return localDisplayProfile; }
-  GfxColorTransform *getXYZ2DisplayTransform();
+  void setDisplayProfile(const GfxLCMSProfilePtr& localDisplayProfileA);
+  GfxLCMSProfilePtr getDisplayProfile() { return localDisplayProfile; }
+  std::shared_ptr<GfxColorTransform> getXYZ2DisplayTransform();
+  int getCmsRenderingIntent();
 #endif
 
   // Add to path.
@@ -1684,12 +1705,11 @@ private:
   GfxState(const GfxState *state, bool copyPath);
 
 #ifdef USE_CMS
-  void *localDisplayProfile;
-  int displayProfileRef;
-  GfxColorTransform *XYZ2DisplayTransformRelCol;
-  GfxColorTransform *XYZ2DisplayTransformAbsCol;
-  GfxColorTransform *XYZ2DisplayTransformSat;
-  GfxColorTransform *XYZ2DisplayTransformPerc;
+  GfxLCMSProfilePtr localDisplayProfile;
+  std::shared_ptr<GfxColorTransform> XYZ2DisplayTransformRelCol;
+  std::shared_ptr<GfxColorTransform> XYZ2DisplayTransformAbsCol;
+  std::shared_ptr<GfxColorTransform> XYZ2DisplayTransformSat;
+  std::shared_ptr<GfxColorTransform> XYZ2DisplayTransformPerc;
 #endif
 };
 
