@@ -24,6 +24,7 @@
 // Copyright (C) 2018 Intevation GmbH <intevation@intevation.de>
 // Copyright (C) 2018, 2020 Adam Reichold <adam.reichold@t-online.de>
 // Copyright (C) 2019, 2020 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2020 Marek Kasik <mkasik@redhat.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -129,6 +130,10 @@ std::unique_ptr<LinkAction> LinkAction::parseAction(const Object *obj, const Goo
   } else if (obj2.isName("Hide")) {
     action = std::make_unique<LinkHide>(obj);
 
+  // ResetForm action
+  } else if (obj2.isName("ResetForm")) {
+    action = std::make_unique<LinkResetForm>(obj);
+
   // unknown action
   } else if (obj2.isName()) {
     action = std::make_unique<LinkUnknown>(obj2.getName());
@@ -396,10 +401,10 @@ LinkDest::LinkDest(const Array *a) {
   // unknown link kind
   } else {
     error(errSyntaxWarning, -1, "Unknown annotation destination type");
+    return;
   }
 
   ok = true;
-  return;
 }
 
 LinkDest::LinkDest(const LinkDest *dest) {
@@ -444,6 +449,8 @@ LinkGoTo::LinkGoTo(const Object *destObj) {
   }
 }
 
+LinkGoTo::~LinkGoTo() = default;
+
 //------------------------------------------------------------------------
 // LinkGoToR
 //------------------------------------------------------------------------
@@ -474,6 +481,8 @@ LinkGoToR::LinkGoToR(Object *fileSpecObj, Object *destObj) {
   }
 }
 
+LinkGoToR::~LinkGoToR() = default;
+
 //------------------------------------------------------------------------
 // LinkLaunch
 //------------------------------------------------------------------------
@@ -552,6 +561,8 @@ LinkURI::LinkURI(const Object *uriObj, const GooString *baseURI) {
   }
 }
 
+LinkURI::~LinkURI() = default;
+
 //------------------------------------------------------------------------
 // LinkNamed
 //------------------------------------------------------------------------
@@ -564,6 +575,8 @@ LinkNamed::LinkNamed(const Object *nameObj) {
   }
 }
 
+LinkNamed::~LinkNamed() = default;
+
 //------------------------------------------------------------------------
 // LinkMovie
 //------------------------------------------------------------------------
@@ -607,6 +620,8 @@ LinkMovie::LinkMovie(const Object *obj) {
   }
 }
 
+LinkMovie::~LinkMovie() = default;
+
 //------------------------------------------------------------------------
 // LinkSound
 //------------------------------------------------------------------------
@@ -645,6 +660,8 @@ LinkSound::LinkSound(const Object *soundObj) {
   }
 }
 
+LinkSound::~LinkSound() = default;
+
 //------------------------------------------------------------------------
 // LinkRendition
 //------------------------------------------------------------------------
@@ -676,7 +693,7 @@ LinkRendition::LinkRendition(const Object *obj) {
         error(errSyntaxWarning, -1, "Invalid Rendition Action: unrecognized operation valued: {0:d}", operationCode);
       } else {
         // retrieve rendition object
-        renditionObj = obj->dictLookup("R");
+        Object renditionObj = obj->dictLookup("R");
         if (renditionObj.isDict()) {
           media = new MediaRendition(&renditionObj);
 	} else if (operationCode == 0 || operationCode == 4) {
@@ -738,6 +755,8 @@ LinkJavaScript::LinkJavaScript(Object *jsObj) {
   }
 }
 
+LinkJavaScript::~LinkJavaScript() = default;
+
 Object LinkJavaScript::createObject(XRef *xref, const GooString &js)
 {
   Dict *linkDict = new Dict(xref);
@@ -793,6 +812,8 @@ LinkOCGState::LinkOCGState(const Object *obj)
   preserveRB = obj->dictLookup("PreserveRB").getBoolWithDefaultValue(true);
 }
 
+LinkOCGState::~LinkOCGState() = default;
+
 //------------------------------------------------------------------------
 // LinkHide
 //------------------------------------------------------------------------
@@ -814,6 +835,44 @@ LinkHide::LinkHide(const Object *hideObj) {
   }
 }
 
+LinkHide::~LinkHide() = default;
+
+//------------------------------------------------------------------------
+// LinkResetForm
+//------------------------------------------------------------------------
+
+LinkResetForm::LinkResetForm(const Object *obj) {
+  Object obj1;
+
+  exclude = false;
+
+  obj1 = obj->dictLookup("Fields");
+  if (obj1.isArray()) {
+    fields.resize(obj1.arrayGetLength());
+    for (int i = 0; i < obj1.arrayGetLength(); ++i) {
+      const Object &obj2 = obj1.arrayGetNF(i);
+      if (obj2.isName())
+        fields[i] = std::string (obj2.getName ());
+      else if (obj2.isRef()) {
+        fields[i] = std::to_string(obj2.getRef().num);
+        fields[i].append(" ");
+        fields[i].append(std::to_string(obj2.getRef().gen));
+        fields[i].append(" R");
+      }
+    }
+  }
+
+  obj1 = obj->dictLookup("Flags");
+  if (obj1.isInt()) {
+    int flags = obj1.getInt();
+
+    if (flags & 0x1)
+      exclude = true;
+  }
+}
+
+LinkResetForm::~LinkResetForm() = default;
+
 //------------------------------------------------------------------------
 // LinkUnknown
 //------------------------------------------------------------------------
@@ -822,6 +881,8 @@ LinkUnknown::LinkUnknown(const char *actionA) {
   action = std::string(actionA ? actionA : "");
 }
 
+LinkUnknown::~LinkUnknown() = default;
+
 //------------------------------------------------------------------------
 // Links
 //------------------------------------------------------------------------
