@@ -15,6 +15,7 @@
  * Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
  * Copyright (C) 2019 Oliver Sander <oliver.sander@tu-dresden.de>
  * Copyright (C) 2019 Alexander Volkov <a.volkov@rusbitech.ru>
+ * Copyright (C) 2020 Philipp Knechtges <philipp-dev@knechtges.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,6 +49,7 @@
 #include <QtCore/QFile>
 #include <QtCore/QByteArray>
 
+#include "poppler-form.h"
 #include "poppler-private.h"
 #include "poppler-page-private.h"
 #include "poppler-outline-private.h"
@@ -647,7 +649,7 @@ namespace Poppler {
     void Document::setColorDisplayProfile(void* outputProfileA)
     {
 #if defined(USE_CMS)
-        GfxColorSpace::setDisplayProfile((cmsHPROFILE)outputProfileA);
+        GfxColorSpace::setDisplayProfile(make_GfxLCMSProfilePtr(outputProfileA));
 #else
         Q_UNUSED(outputProfileA);
 #endif
@@ -667,7 +669,7 @@ namespace Poppler {
     void* Document::colorRgbProfile() const
     {
 #if defined(USE_CMS)
-        return (void*)GfxColorSpace::getRGBProfile();
+        return GfxColorSpace::getRGBProfile().get();
 #else
         return nullptr;
 #endif
@@ -676,7 +678,7 @@ namespace Poppler {
     void* Document::colorDisplayProfile() const
     {
 #if defined(USE_CMS)
-       return (void*)GfxColorSpace::getDisplayProfile();
+       return GfxColorSpace::getDisplayProfile().get();
 #else
        return nullptr;
 #endif
@@ -829,6 +831,21 @@ namespace Poppler {
         return result;
     }
 
+    QVector<FormFieldSignature*> Document::signatures() const
+    {
+        QVector<FormFieldSignature*> result;
+
+        const std::vector<::FormFieldSignature*> pSignatures = m_doc->doc->getSignatureFields();
+
+        for (::FormFieldSignature *pSignature : pSignatures) {
+            ::FormWidget *fw = pSignature->getWidget(0);
+            ::Page *p = m_doc->doc->getPage(fw->getWidgetAnnotation()->getPageNum());
+            result.append(new FormFieldSignature(m_doc, p, static_cast<FormWidgetSignature*>(fw)));
+        }
+
+        return result;
+    }
+
     QDateTime convertDate( const char *dateString )
     {
         int year, mon, day, hour, min, sec, tzHours, tzMins;
