@@ -1,6 +1,6 @@
 /* poppler-form.h: qt interface to poppler
  * Copyright (C) 2007-2008, Pino Toscano <pino@kde.org>
- * Copyright (C) 2008, 2011, 2016, 2017, 2019, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2008, 2011, 2016, 2017, 2019, 2020, Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2012, Adam Reichold <adamreichold@myopera.com>
  * Copyright (C) 2016, Hanno Meyer-Thurow <h.mth@web.de>
  * Copyright (C) 2017, Hans-Ulrich Jüttner <huj@froreich-bioscientia.de>
@@ -28,7 +28,8 @@
 #ifndef _POPPLER_QT5_FORM_H_
 #define _POPPLER_QT5_FORM_H_
 
-#include <time.h>
+#include <memory>
+#include <ctime>
 #include <QtCore/QDateTime>
 #include <QtCore/QList>
 #include <QtCore/QRectF>
@@ -55,7 +56,7 @@ namespace Poppler {
     /**
 	 The class containing the appearance information
 
-	 \since 0.78
+	 \since 0.79
      */
 
     class POPPLER_QT5_EXPORT FormFieldIcon {
@@ -80,6 +81,9 @@ namespace Poppler {
       \since 0.6
      */
     class POPPLER_QT5_EXPORT FormField {
+
+    friend class FormFieldData;
+
     public:
 
 	/**
@@ -202,9 +206,9 @@ namespace Poppler {
 
     protected:
 	/// \cond PRIVATE
-	FormField(FormFieldData &dd);
+	FormField(std::unique_ptr<FormFieldData> dd);
 
-	FormFieldData *m_formData;
+	std::unique_ptr<FormFieldData> m_formData;
 	/// \endcond
 
     private:
@@ -232,7 +236,7 @@ namespace Poppler {
 	/// \cond PRIVATE
 	FormFieldButton(DocumentData *doc, ::Page *p, ::FormWidgetButton *w);
 	/// \endcond
-	~FormFieldButton();
+	~FormFieldButton() override;
 
 	FormType type() const override;
 
@@ -249,7 +253,7 @@ namespace Poppler {
 	/**
 	 * Gets the icon used by the button
 	 *
-	 * \since 0.78
+	 * \since 0.79
 	 */
 	FormFieldIcon icon() const;
 
@@ -257,7 +261,7 @@ namespace Poppler {
 	 * Sets a new icon for the button, it has to be a icon 
 	 * returned by FormFieldButton::icon.
 	 *
-	 * \since 0.78
+	 * \since 0.79
 	 */ 
 	void setIcon(const FormFieldIcon &icon);
 
@@ -303,7 +307,7 @@ namespace Poppler {
 	/// \cond PRIVATE
 	FormFieldText(DocumentData *doc, ::Page *p, ::FormWidgetText *w);
 	/// \endcond
-	~FormFieldText();
+	~FormFieldText() override;
 
 	FormType type() const override;
 
@@ -323,6 +327,13 @@ namespace Poppler {
 	 */
 	void setText( const QString& text );
 
+	/**
+	  Sets the text inside the Appearance Stream to the specified
+	  \p text
+	  \since 0.80
+	 */
+	void setAppearanceText( const QString& text );
+
 	/**
 	  Whether this text field is a password input, eg its text \b must be
 	  replaced with asterisks.
@@ -385,7 +396,7 @@ namespace Poppler {
 	/// \cond PRIVATE
 	FormFieldChoice(DocumentData *doc, ::Page *p, ::FormWidgetChoice *w);
 	/// \endcond
-	~FormFieldChoice();
+	~FormFieldChoice() override;
 
 	FormType type() const override;
 
@@ -399,6 +410,14 @@ namespace Poppler {
 	 */
 	QStringList choices() const;
 
+	/**
+	  The possible choices of the choice field.
+	  The first value of the pair is the display name of the choice,
+	  The second value is the export value (i.e. for use in javascript, etc) of the choice
+	  @since 0.87
+	 */
+	QVector<QPair<QString,QString>> choicesWithExportValues() const;
+
 	/**
 	  Whether this FormFieldChoice::ComboBox is editable, i.e. the user
 	  can type in a custom value.
@@ -745,7 +764,7 @@ namespace Poppler {
 	/// \cond PRIVATE
 	FormFieldSignature(DocumentData *doc, ::Page *p, ::FormWidgetSignature *w);
 	/// \endcond
-	~FormFieldSignature();
+	~FormFieldSignature() override;
 
 	FormType type() const override;
 
