@@ -1,7 +1,7 @@
 /* poppler-qt.h: qt interface to poppler
  * Copyright (C) 2005, Net Integration Technologies, Inc.
  * Copyright (C) 2005, 2007, Brad Hards <bradh@frogmouth.net>
- * Copyright (C) 2005-2015, 2017-2019, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2005-2015, 2017-2020, Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2005, Stefan Kebekus <stefan.kebekus@math.uni-koeln.de>
  * Copyright (C) 2006-2011, Pino Toscano <pino@kde.org>
  * Copyright (C) 2009 Shawn Rutledge <shawn.t.rutledge@gmail.com>
@@ -21,6 +21,7 @@
  * Copyright (C) 2018 Nelson Benítez León <nbenitezl@gmail.com>
  * Copyright (C) 2019 Jan Grulich <jgrulich@redhat.com>
  * Copyright (C) 2019 Alexander Volkov <a.volkov@rusbitech.ru>
+ * Copyright (C) 2020 Philipp Knechtges <philipp-dev@knechtges.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,6 +67,7 @@ namespace Poppler {
     class PageData;
 
     class FormField;
+    class FormFieldSignature;
 
     class TextBoxData;
 
@@ -1189,7 +1191,10 @@ delete it;
 
 	  \param outputProfileA is a \c cmsHPROFILE of the LCMS library.
 
-	  \note This should be called before any rendering happens and only once during the lifetime of the current process.
+	  \note This should be called before any rendering happens.
+
+	  \note It is assumed that poppler takes over the owernship of the corresponding cmsHPROFILE. In particular,
+          it is no longer the caller's responsibility to close the profile after use.
 
 	   \since 0.12
 	*/
@@ -1209,6 +1214,9 @@ delete it;
 
 	  \return a \c cmsHPROFILE of the LCMS library.
 
+	  \note The returned profile stays a property of poppler and shall NOT be closed by the user. It's
+	  existence is guaranteed for as long as this instance of the Document class is not deleted.
+
 	   \since 0.12
 	*/
 	void* colorRgbProfile() const;
@@ -1217,6 +1225,9 @@ delete it;
 
 	  \return a \c cmsHPROFILE of the LCMS library.
 
+	  \note The returned profile stays a property of poppler and shall NOT be closed by the user. It's
+	  existence is guaranteed for as long as this instance of the Document class is not deleted.
+
 	   \since 0.12
 	*/
 	void *colorDisplayProfile() const;
@@ -1846,6 +1857,16 @@ QString subject = m_doc->info("Subject");
 	*/
 	QVector<int> formCalculateOrder() const;
 
+	/**
+	 Returns the signatures of this document.
+
+	 Prefer to use this over getting the signatures for all the pages of the document
+	 since there are documents with signatures that don't belong to a given page
+
+	 \since 0.88
+	*/
+	QVector<FormFieldSignature*> signatures() const;
+
 	/**
 	   Destructor.
 	*/
