@@ -32,8 +32,10 @@
 // Copyright (C) 2016 Jason Crain <jason@aquaticape.us>
 // Copyright (C) 2018 Martin Packman <gzlist@googlemail.com>
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
-// Copyright (C) 2019 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2019, 2020 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2019 Kris Jurka <jurka@ejurka.com>
+// Copyright (C) 2020 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2020 Philipp Knechtges <philipp-dev@knechtges.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -287,16 +289,16 @@ static cairo_antialias_t antialiasEnum = CAIRO_ANTIALIAS_DEFAULT;
 #ifdef USE_CMS
 static unsigned char *icc_data;
 static int icc_data_size;
-static cmsHPROFILE profile;
+static GfxLCMSProfilePtr profile;
 #endif
 
-struct AntiliasOption
+struct AntialiasOption
 {
   const char *name;
   cairo_antialias_t value;
 };
 
-static const AntiliasOption antialiasOptions[] =
+static const AntialiasOption antialiasOptions[] =
 {
   { "default",  CAIRO_ANTIALIAS_DEFAULT },
   { "none",     CAIRO_ANTIALIAS_NONE },
@@ -310,7 +312,7 @@ static const AntiliasOption antialiasOptions[] =
 
 static bool parseAntialiasOption()
 {
-  const AntiliasOption *option = antialiasOptions;
+  const AntialiasOption *option = antialiasOptions;
   while (option->name) {
     if (antialias.cmp(option->name) == 0) {
       antialiasEnum = option->value;
@@ -412,7 +414,7 @@ static void writePageImage(GooString *filename)
       cmsUInt8Number profileID[17];
       profileID[16] = '\0';
 
-      cmsGetHeaderProfileID(profile,profileID);
+      cmsGetHeaderProfileID(profile.get(),profileID);
       static_cast<PNGWriter*>(writer)->setICCProfile(reinterpret_cast<char *>(profileID), icc_data, icc_data_size);
     } else {
       static_cast<PNGWriter*>(writer)->setSRGBProfile();
@@ -1127,13 +1129,13 @@ int main(int argc, char *argv[]) {
       exit(4);
     }
     fclose(file);
-    profile = cmsOpenProfileFromMem(icc_data, icc_data_size);
+    profile = make_GfxLCMSProfilePtr(cmsOpenProfileFromMem(icc_data, icc_data_size));
     if (!profile) {
       fprintf(stderr, "Error: lcms error opening profile\n");
       exit(4);
     }
   } else {
-    profile = cmsCreate_sRGBProfile();
+    profile = make_GfxLCMSProfilePtr(cmsCreate_sRGBProfile());
   }
   GfxColorSpace::setDisplayProfile(profile);
 #endif
@@ -1175,8 +1177,8 @@ int main(int argc, char *argv[]) {
   // outputting a single page, ensure that even/odd page selection doesn't
   // filter out that single page.
   if (firstPage == lastPage &&
-       ((printOnlyEven && firstPage % 2 == 0) ||
-        (printOnlyOdd && firstPage % 2 == 1))) {
+       ((printOnlyEven && firstPage % 2 == 1) ||
+        (printOnlyOdd && firstPage % 2 == 0))) {
     fprintf(stderr, "Invalid even/odd page selection, no pages match criteria.\n");
     exit(99);
   }
@@ -1285,7 +1287,6 @@ int main(int argc, char *argv[]) {
     delete userPW;
 
 #ifdef USE_CMS
-  cmsCloseProfile(profile);
   if (icc_data)
     gfree(icc_data);
 #endif
